<?php

/**
 * Copyright  2008 mavericck
 *
 * This file is part of Sitemap Generator.
 *
 * Sitemap Generator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Sitemap Generator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Sitemap Generator.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

function sg_check_enviroment()
{
    global $lang, $page;

    // 1. check if we can create the sitemap index in the root
    //    if not, die here and warn the admin!
    if (!file_exists(MYBB_ROOT . 'sitemap.xml'))
    {
        $works = @fopen(MYBB_ROOT . 'sitemap.xml', 'w');
        @fclose($works);
        if (!$works)
        {
            $page->add_breadcrumb_item($lang->sitemap_generator);
            $page->output_header($lang->sitemap_generator_error);
            $page->output_inline_error(array($lang->error_sitemaps_index_create));
            $page->output_footer();
            die();
        }
        if (!is_writable(MYBB_ROOT . 'sitemap.xml'))
        {
            $page->add_breadcrumb_item($lang->sitemap_generator);
            $page->output_header($lang->sitemap_generator_error);
            $page->output_inline_error(array($lang->error_sitemaps_index_write));
            $page->output_footer();
            die();
        }
    }

    // 2. check if bad permissions have forced the script to create the sitemaps/ folder
    //    If so, warn the admin!
    if (file_exists(MYBB_ROOT . 'sitemaps'))
    {
        $list = array();

        if ($handle = opendir(MYBB_ROOT . 'sitemaps'))
        {
            while (false !== ($file = readdir($handle)))
            {
                if (preg_match('#^sitemap_(threads|forums)_[0-9]+\.xml$#', $file))
                {
                    $list[] = array('name'     => $file);
                }
            }
        }
        @closedir($handle);

        if (count($list) > 0)
        {
            flash_message($lang->permission_warn, 'error');
        }
    }

    // 3. check if we have a robots.txt file
    if (!file_exists(MYBB_ROOT . 'robots.txt'))
    {
        flash_message($lang->robots_not_found, 'error');
    }

    // 4. check if we have the sitemap line in the robots.txt
    if (file_exists(MYBB_ROOT . 'robots.txt'))
    {
        $handle = @fopen(MYBB_ROOT . 'robots.txt', 'r');
        if ($handle != false)
        {
            if (filesize(MYBB_ROOT . 'robots.txt') == 0)
            {
                flash_message($lang->robots_sitemap_line, 'error');
            }
            else
            {
                $contents = fread($handle, filesize(MYBB_ROOT . 'robots.txt'));
                if (strstr($contents, 'Sitemap') == false)
                {
                    flash_message($lang->robots_sitemap_line, 'error');
                }
            }
        }
        fclose($handle);
    }

    return true;
}

function sg_get_files($type = 'threads')
{
    $list = array();

    if ($handle = opendir(MYBB_ROOT))
    {
        while (false !== ($file = readdir($handle)))
        {
            if (preg_match('#^sitemap_' . $type . '_[0-9]+\.xml$#', $file))
            {
                $list[] = array('name'     => $file,
                                'size'     => filesize(MYBB_ROOT . $file),
                                'date'     => filemtime(MYBB_ROOT . $file),
                );
            }
        }
    }

    closedir($handle);
    clearstatcache();
    return $list;
}

function sg_build_index_file_table()
{
    global $lang, $settings;

    $table = new Table();
    $table->construct_header($lang->file_name, array('width' => '15%'));
	$table->construct_header($lang->file_size, array('width' => '15%'));
	$table->construct_header($lang->file_created, array('width' => '15%'));
	$table->construct_header($lang->controls, array('class' => 'align_center'));

	$table->construct_cell('sitemap.xml', array());
	$table->construct_cell(get_friendly_size(filesize('../sitemap.xml'), array()));
	$table->construct_cell(date('j M Y H:i', filemtime('../sitemap.xml')), array());
	$table->construct_cell('<a href="' . $settings['bburl'] . '/sitemap.xml">' . $lang->view . '</a> | ' .
	                       '<a href="index.php?module=forum/sg_status&amp;type=download&amp;name=sitemap.xml">' . $lang->download . '</a> | ' .
	                       '<a href="index.php?module=forum/sg_status&amp;type=rebuild_index">' . $lang->rebuild_index . '</a>',
	                       array('class' => 'align_center'));

	    $table->construct_row();

	$table->output($lang->index_file);
}

function sg_build_status_table($list)
{
    global $lang, $settings;

    $table = new Table();
    $table->construct_header($lang->file_name, array('width' => '35%'));
	$table->construct_header($lang->file_size, array('width' => '15%'));
	$table->construct_header($lang->file_created, array('width' => '15%'));
	$table->construct_header($lang->controls, array('class' => 'align_center'));

	foreach ($list as $item)
	{
	    $table->construct_cell($item['name'], array());
	    $table->construct_cell(get_friendly_size($item['size']), array());
	    $table->construct_cell(date('j M Y H:i', $item['date']), array());
	    $table->construct_cell('<a href="' . $settings['bburl'] . '/' . $item['name'] . '">' . $lang->view . '</a> | ' .
	                           '<a href="index.php?module=forum/sg_status&amp;type=download&amp;name=' . $item['name'] . '">' . $lang->download . '</a> | ' .
	                           '<a href="index.php?module=forum/sg_status&amp;type=delete&amp;name=' . $item['name'] . '">' . $lang->delete . '</a>',
	                           array('class' => 'align_center'));

	    $table->construct_row();
	}

	$table->output($lang->files_list);
}

function sg_download_file($name)
{
    global $lang, $page, $settings;

    if (preg_match('#^sitemap_(threads|forums|announcements|profiles)_[0-9]+\.xml$#', $name))
    {
        header('Content-Encoding: UTF-8');
        header('Content-Type: text/xml');
		header('Content-Disposition: attachment; filename="' . $name . '"');
		echo file_get_contents($settings['bburl'] . '/' . $name);
		exit;
    }
    else if (preg_match('#^sitemap.xml$#', $name))
    {
        header('Content-Encoding: UTF-8');
        header('Content-Type: text/xml');
		header('Content-Disposition: attachment; filename="sitemap.xml"');
		echo file_get_contents($settings['bburl'] . '/sitemap.xml');
		exit;
    }
    else
    {
        $page->add_breadcrumb_item($lang->sitemap_generator);
        $page->output_header($lang->sitemap_generator_error);
        $page->output_inline_error(array($lang->error_invalid_file));
        $page->output_footer();
        return;
    }
}

function sg_delete_file($name)
{
    global $lang, $page, $settings;

    if (preg_match('#^sitemap_(threads|forums|announcements|profiles)_[0-9]+\.xml$#', $name))
    {
        unlink('../' . $name);
    }
    else
    {
        $page->add_breadcrumb_item($lang->sitemap_generator);
        $page->output_header($lang->sitemap_generator_error);
        $page->output_inline_error(array($lang->error_invalid_file));
        $page->output_footer();
        return;
    }
}

function sg_fetch_threads()
{
    global $db, $mybb;

    $list = array();

    $forum_permissions = forum_permissions(0, 0, 1);

    if ($mybb->settings['sg_threads_pis_enable'] == 1)
    {
        $pis_field = ($mybb->settings['sg_threads_pis_criterion'] == 'replies') ? ', replies AS pis_index' : ', views AS pis_index';
        $pis_index_max = 0;
        $pis_index_min = 999999;
    }
    else
    {
        $pis_field = '';
    }

    if((int) $mybb->settings['sg_threads_cutoff'] > 0)
    {
        $cutoff_date = time() - ($mybb->settings['sg_threads_cutoff'] * 24 * 60 * 60);
        $cutoff = ' AND lastpost > ' . $cutoff_date;
    }
    else
    {
        $cutoff = '';
    }

    if ($mybb->settings['sg_threads_clause'] != '')
    {
        $custom_clause = ' AND (' . $mybb->settings['sg_threads_clause'] . ')';
    }
    else
    {
        $custom_clause = '';
    }

    $query = $db->simple_select('threads', 'fid, tid, lastpost' . $pis_field, 'visible = 1' . $cutoff . $custom_clause);

    while ($row = $db->fetch_array($query))
    {
        if (!isset($forum_permissions[$row['fid']]['canview']) || $forum_permissions[$row['fid']]['canview'] == 1)
        {
            // prepare loc
            if($mybb->settings['seourls'] == 'yes' || ($mybb->settings['seourls'] == 'auto' && $_SERVER['SEO_SUPPORT'] == 1))
            {
                $loc = $mybb->settings['bburl'] . '/thread-' . $row['tid'] . '.html';
            }
            else
            {
                $loc = $mybb->settings['bburl'] . '/showthread.php?tid=' . $row['tid'];
            }

            // prepare priority
            $priority = ($mybb->settings['sg_threads_pis_enable'] == 1) ? $row['pis_index'] : '0.5';
            if ($mybb->settings['sg_threads_pis_enable'] == 1)
            {
                $pis_index_max = ($row['pis_index'] > $pis_index_max) ? $row['pis_index'] : $pis_index_max;
                $pis_index_min = ($row['pis_index'] < $pis_index_min) ? $row['pis_index'] : $pis_index_min;
            }

            $list[] = array('loc'        => $loc,
                            'lastmod'    => date('Y-m-d', $row['lastpost']),
                            'changefreq' => $mybb->settings['sg_threads_chfreq'],
                            'priority'   => $priority
            );
        }
    }

    if ($mybb->settings['sg_threads_pis_enable'] == 1)
    {
        foreach ($list as $key => $item)
        {
            $list[$key]['priority'] = sg_pis_scale($item['priority'], $pis_index_min, $pis_index_max);
        }
        if (SG_PIS_LOW_PASS_NUMERIC_FILTER)
        {
            $prev = 0.5;
            $pole = 0.8;
            foreach ($list as $key => $item)
            {
                $list[$key]['priority'] = round(($pole * $item['priority']) + ((1 - $pole) * $prev), 1);
                $prev = $item['priority'];
            }
        }
    }

    return $list;
}

function sg_pis_scale($value, $v_min, $v_max, $def_min = 0.4, $def_max = 1)
{
    $gain = ($v_max - $v_min) / ($def_max - $def_min);
    if ($gain == 0) $gain = 1;
    $scaled_v_min = $v_min / $gain;
    $offset = $scaled_v_min - $def_min;
    $priority = round(($value / $gain) - $offset, 1);

    return $priority;
}

function sg_fetch_announcements()
{
    global $db, $mybb;

    $list = array();

    if((int) $mybb->settings['sg_announcements_cutoff'] > 0)
    {
        $cutoff_date = time() - ($mybb->settings['sg_announcements_cutoff'] * 24 * 60 * 60);
        $cutoff = ' AND startdate > ' . $cutoff_date;
    }
    else
    {
        $cutoff = '';
    }

    if ($mybb->settings['sg_announcements_clause'] != '')
    {
        $custom_clause = ' AND (' . $mybb->settings['sg_announcements_clause'] . ')';
    }
    else
    {
        $custom_clause = '';
    }

    $query = $db->simple_select('announcements', 'aid, startdate', '1 = 1' . $cutoff . $custom_clause);

    while ($row = $db->fetch_array($query))
    {
        // prepare loc
        if($mybb->settings['seourls'] == 'yes' || ($mybb->settings['seourls'] == 'auto' && $_SERVER['SEO_SUPPORT'] == 1))
        {
            $loc = $mybb->settings['bburl'] . '/announcement-' . $row['aid'] . '.html';
        }
        else
        {
            $loc = $mybb->settings['bburl'] . '/announcements.php?aid=' . $row['aid'];
        }

        $list[] = array('loc'        => $loc,
        'lastmod'    => date('Y-m-d', $row['startdate']),
        'changefreq' => $mybb->settings['sg_announcements_chfreq'],
        );
    }
    return $list;
}

function sg_fetch_profiles()
{
    global $db, $mybb;

    $list = array();

    if((int) $mybb->settings['sg_profiles_cutoff'] > 0)
    {
        $cutoff_date = time() - ($mybb->settings['sg_profiles_cutoff'] * 24 * 60 * 60);
        $cutoff = ' AND lastactive > ' . $cutoff_date;
    }
    else
    {
        $cutoff = '';
    }

    if ($mybb->settings['sg_profiles_clause'] != '')
    {
        $custom_clause = ' AND (' . $mybb->settings['sg_profiles_clause'] . ')';
    }
    else
    {
        $custom_clause = '';
    }

    $query = $db->simple_select('users', 'uid', '1 = 1' . $cutoff . $custom_clause);

    while ($row = $db->fetch_array($query))
    {
        if($mybb->settings['seourls'] == 'yes' || ($mybb->settings['seourls'] == 'auto' && $_SERVER['SEO_SUPPORT'] == 1))
        {
            $loc = $mybb->settings['bburl'] . '/user-' . $row['uid'] . '.html';
        }
        else
        {
            $loc = $mybb->settings['bburl'] . '/member.php?action=profile&amp;uid=' . $row['uid'];
        }

        $list[] = array('loc'        => $loc,
                        'changefreq' => $mybb->settings['sg_profiles_chfreq'],
        );
    }
    return $list;
}

function sg_fetch_forums()
{
    global $db, $mybb;

    $list = array();

    $forum_permissions = forum_permissions(0, 0, 1);

    if ($mybb->settings['sg_forums_pis_enable'] == 1)
    {
        $pis_field = ($mybb->settings['sg_forums_pis_criterion'] == 'threads') ? ', threads AS pis_index' : ', posts AS pis_index';
        $pis_index_max = 0;
        $pis_index_min = 999999;
    }
    else
    {
        $pis_field = '';
    }

    if((int) $mybb->settings['sg_forums_cutoff'] > 0)
    {
        $cutoff_date = time() - ($mybb->settings['sg_forums_cutoff'] * 24 * 60 * 60);
        $cutoff = ' AND lastpost > ' . $cutoff_date;
    }
    else
    {
        $cutoff = '';
    }

    if ($mybb->settings['sg_forums_clause'] != '')
    {
        $custom_clause = ' AND (' . $mybb->settings['sg_forums_clause'] . ')';
    }
    else
    {
        $custom_clause = '';
    }

    $query = $db->simple_select('forums', 'fid, lastpost' . $pis_field, "active = 1 AND type = 'f' AND password = ''" . $cutoff . $custom_clause);

    while ($row = $db->fetch_array($query))
    {
        if (!isset($forum_permissions[$row['fid']]['canview']) || $forum_permissions[$row['fid']]['canview'] == 1)
        {
            // prepare loc
            if($mybb->settings['seourls'] == 'yes' || ($mybb->settings['seourls'] == 'auto' && $_SERVER['SEO_SUPPORT'] == 1))
            {
                $loc = $mybb->settings['bburl'] . '/forum-' . $row['fid'] . '.html';
            }
            else
            {
                $loc = $mybb->settings['bburl'] . '/forumdisplay.php?fid=' . $row['fid'];
            }

            // prepare priority
            $priority = ($mybb->settings['sg_forums_pis_enable'] == 1) ? $row['pis_index'] : '0.5';
            if ($mybb->settings['sg_forums_pis_enable'] == 1)
            {
                $pis_index_max = ($row['pis_index'] > $pis_index_max) ? $row['pis_index'] : $pis_index_max;
                $pis_index_min = ($row['pis_index'] < $pis_index_min) ? $row['pis_index'] : $pis_index_min;
            }

            $list[] = array('loc'        => $loc,
                            'lastmod'    => date('Y-m-d', $row['lastpost']),
                            'changefreq' => $mybb->settings['sg_forums_chfreq'],
                            'priority'   => $priority
            );
        }
    }

    if ($mybb->settings['sg_forums_pis_enable'] == 1)
    {
        foreach ($list as $key => $item)
        {
            $list[$key]['priority'] = sg_pis_scale($item['priority'], $pis_index_min, $pis_index_max);
        }
        if (SG_PIS_LOW_PASS_NUMERIC_FILTER)
        {
            $prev = 0.5;
            $pole = 0.8;
            foreach ($list as $key => $item)
            {
                $list[$key]['priority'] = round(($pole * $item['priority']) + ((1 - $pole) * $prev), 1);
                $prev = $item['priority'];
            }
        }
    }

    return $list;
}

function sg_make_sitemap($type, $list, $threshold = 49995, $suppress_errors = false)
{
    global $lang;

    $counter1 = 0;
    $counter2 = 0;

    if (count($list) < 1)
    {
        if (!$suppress_errors)
        {
            flash_message($lang->error_no_data, 'error');
        }
        return false;
    }

    while ($row = array_pop($list))
    {
        $counter1++;
        $buffer[] = $row;

        if ($counter1 == $threshold)
        {
            $counter2++;
            sg_write_sitemap($type, $buffer, $counter2);
            unset($buffer);
            $counter1 = 0;
        }
    }

    if (count($buffer) > 0)
    {
        $counter2++;
        sg_write_sitemap($type, $buffer, $counter2);
        unset($buffer);
    }

    sg_rebuild_index();

    return true;
}

function sg_rebuild_index()
{
    global $mybb;

    $list = array();

    if ($handle = opendir(MYBB_ROOT))
    {
        while (false !== ($file = readdir($handle)))
        {
            if (preg_match('#^sitemap_(threads|forums|announcements|profiles)_[0-9]+\.xml$#', $file))
            {
                $list[] = array('loc'        => $mybb->settings['bburl'] . '/' . $file,
                                'lastmod'    => date('Y-m-d', filemtime(MYBB_ROOT . $file))
                                );
            }
        }
    }
    closedir($handle);
    clearstatcache();

    if (count($list) < 1)
    {
        $content = '';
    }
    else
    {
        foreach ($list as $item)
        {
            $content .= sg_wrap_index_item($item);
        }
    }

    $content = sg_wrap_index_file($content);

    $handle = fopen(MYBB_ROOT . 'sitemap.xml', 'w');
    fwrite($handle, $content, strlen($content));
    fclose($handle);
}

function sg_update_settings($settings)
{
    global $db;

    $settings = sg_clean_settings($settings);

    foreach($settings as $name => $value)
    {
        $db->update_query('settings', array('value' => $db->escape_string($value)), "name='" . $db->escape_string($name) . "'");
    }
    rebuild_settings();
}

function sg_clean_settings($settings)
{
    foreach ($settings as $key => $value)
    {
        if (strstr($key, 'chfreq'))
        {
            if (!in_array($value, array('always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never')))
            {
                $settings[$key] = 'never';
            }
        }
        if (strstr($key, 'pis_enable'))
        {
            if (!in_array((int) $value, array(0, 1)))
            {
                $settings[$key] = 0;
            }
        }
        if (strstr($key, 'threads_pis_criterion'))
        {
            if (!in_array($value, array('replies', 'views')))
            {
                $settings[$key] = 'replies';
            }
        }
        if (strstr($key, 'forums_pis_criterion'))
        {
            if (!in_array($value, array('threads', 'posts')))
            {
                $settings[$key] = 'threads';
            }
        }
        if (strstr($key, 'threshold'))
        {
            $settings[$key] = (int) $value;
        }
        if (strstr($key, 'cutoff'))
        {
            $settings[$key] = (int) $value;
        }
        if (strstr($key, 'clause') && !SG_ENABLE_CUSTOM_SQL_CLAUSE)
        {
            $settings[$key] = '';
        }
    }

    return $settings;
}

function sg_write_sitemap($type, $list, $counter)
{
    global $lang, $page;

    foreach ($list as $item)
    {
        $content .= sg_wrap_item($item);
    }
    unset($list);
    $content = sg_wrap_file($content);

    // try to write files in root folder
    $fp = @fopen(MYBB_ROOT . 'sitemap_' . $type . '_' . $counter . '.xml', 'w');
    if ($fp == false)
    {
        // we have not permissions to write into the root
        // fallback: try to create a "sitemaps" folder

        $sitemaps_folder = sg_create_sitemaps_folder();
        if ($sitemaps_folder == false)
        {
            // we can't even create a sitemaps folder. Die here!
            $page->add_breadcrumb_item($lang->sitemap_generator);
            $page->output_header($lang->sitemap_generator_error);
            $page->output_inline_error(array($lang->error_sitemaps_folder));
            $page->output_footer();
            return false;
        }

        $fp = @fopen(MYBB_ROOT . 'sitemaps/sitemap_' . $type . '_' . $counter . '.xml', 'w');
        if ($fp == false)
        {
            // we have the folder, but we can't create files into it. Die here!
            $page->add_breadcrumb_item($lang->sitemap_generator);
            $page->output_header($lang->sitemap_generator_error);
            $page->output_inline_error(array($lang->error_file_open));
            $page->output_footer();
            return false;
        }
    }

    $works = @fwrite($fp, $content, strlen($content));
    if ($works == false)
    {
        $page->add_breadcrumb_item($lang->sitemap_generator);
        $page->output_header($lang->sitemap_generator_error);
        $page->output_inline_error(array($lang->error_file_write));
        $page->output_footer();
        return false;
    }
    @fclose($fp);
}

function sg_create_sitemaps_folder()
{
    if (!file_exists(MYBB_ROOT . 'sitemaps'))
    {
        $works = @mkdir(MYBB_ROOT . 'sitemaps', 0777);
        if (!$works)
        {
            return false;
        }
        return true;
    }
    return true;
}

function sg_wrap_item($item)
{
    $output  = ' <url>'. "\n";
    $output .= '  <loc>' . $item['loc'] . '</loc>'. "\n";
    $output .= (key_exists('lastmod', $item)) ? ('  <lastmod>' . $item['lastmod'] . '</lastmod>' . "\n") : '';
    $output .= (key_exists('changefreq', $item)) ? ('  <changefreq>' . $item['changefreq'] . '</changefreq>' . "\n") : '';
    $output .= (key_exists('priority', $item)) ? ('  <priority>' . $item['priority'] . '</priority>' . "\n") : '';
    $output .= ' </url>'. "\n";

    return $output;
}

function sg_wrap_index_item($item)
{
    $output  = ' <sitemap>'. "\n";
    $output .= '  <loc>' . $item['loc'] . '</loc>'. "\n";
    $output .= (key_exists('lastmod', $item) == true) ? ('  <lastmod>' . $item['lastmod'] . '</lastmod>' . "\n") : '';
    $output .= ' </sitemap>'. "\n";

    return $output;
}

function sg_wrap_index_file($content)
{
    $output  = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
    $output .= '<sitemapindex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' . "\n";
    $output .= ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9' . "\n";
    $output .= ' http://www.sitemaps.org/schemas/sitemap/0.9/siteindex.xsd"' . "\n";
    $output .= ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n" . $content . '</sitemapindex>';

    return $output;
}

function sg_wrap_file($content)
{
    $output  = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
    $output .= '<urlset xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' . "\n";
    $output .= ' xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9' . "\n";
    $output .= ' http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd"' . "\n";
    $output .= ' xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n" . $content . '</urlset>';

    return $output;
}

function sg_build_ping_table()
{
    global $lang, $settings;

    $built_in = array ('Google'   => 'http://www.google.com/webmasters/sitemaps/ping?sitemap=',
                       'Yahoo_1'  => 'http://search.yahooapis.com/SiteExplorerService/V1/updateNotification?appid=SitemapWriter&url=',
        			   'Yahoo_2'  => 'http://search.yahooapis.com/SiteExplorerService/V1/ping?sitemap=',
                       'Ask'      => 'http://submissions.ask.com/ping?sitemap=',
                       'Moreover' => 'http://api.moreover.com/ping?u=',
                       'Live.com' => 'http://webmaster.live.com/ping.aspx?siteMap='
                       );

    $table = new Table();
    $table->construct_header($lang->service_name, array('width' => '35%'));
	$table->construct_header($lang->submit);

	foreach ($built_in as $name => $url)
	{
	    $table->construct_cell($name, array());
	    $table->construct_cell('<a href="' . $url . $settings['bburl'] . '/sitemap.xml" target="_blank">Submit the sitemap index file to this service</a>', array());

	    $table->construct_row();
	}

	$table->output($lang->built_in_services);

	$table = new Table();
    $table->construct_header($lang->service_name, array('width' => '35%'));
	$table->construct_header($lang->submit, array('width' => '35%'));
	$table->construct_header($lang->controls, array('class' => 'align_center'));

	$custom = @unserialize($settings['sg_custom_services']);
	if ($custom == false || count($custom) < 1)
	{
	    $table->construct_cell($lang->none);
	    $table->construct_cell('');
	    $table->construct_cell('');

	    $table->construct_row();
	}
	else
	{
	    foreach ($custom as $key => $service)
	    {
	        $table->construct_cell($service['name'], array());
	        $table->construct_cell('<a href="' . $service['url'] . $settings['bburl'] . '/sitemap.xml" target="_blank">Submit the sitemap index file to this service</a>', array());
	        $table->construct_cell('<a href="index.php?module=forum/sg_ping&amp;action=delService&amp;key=' . $key . '">Remove service</a>', array('class' => 'align_center'));

	        $table->construct_row();
	    }
	}

	$table->output($lang->custom_services);

	$form = new Form('index.php?module=forum/sg_ping&amp;action=addService', 'post');

	$form_container = new FormContainer($lang->add_service);

	$form_container->output_row($lang->service_name, $lang->service_name_desc, $form->generate_text_box('sg_service_name'));
	$form_container->output_row($lang->service_url, $lang->service_url_desc, $form->generate_text_box('sg_service_url'));
	$form_container->end();

	$buttons[] = $form->generate_submit_button($lang->add_service);
	$form->output_submit_wrapper($buttons);
	$form->end();
}
?>